------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Paul Leroy
--                     Mail : paul.leroy@lpp.polytechnique.fr
----------------------------------------------------------------------------
library IEEE;
use IEEE.numeric_std.all;
use IEEE.std_logic_1164.all;

library staging_lpp;
use staging_lpp.PLE_lpp_fft.all;

entity  TestBench_BUTTERFLY_TOP is 

end TestBench_BUTTERFLY_TOP;

architecture TestBench_BUTTERFLY_TOP of TestBench_BUTTERFLY_TOP is

constant    Sample_SZ   :   integer :=  16;

signal      clk         :   std_logic:='0';
signal      rstn        :   std_logic:='0';

signal      Are    :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');
signal      Aim    :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');
signal      Bre    :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');
signal      Bim    :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');
signal      c      :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');
signal      cps    :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');
signal      cms    :   std_logic_vector( Sample_SZ-1 downto 0 ) := ( others => '0');

signal      Resultat    :   std_logic_vector( 2*Sample_SZ-1 downto 0 );

signal      sel_out     :   std_logic_vector( 3 downto 0 );

signal      sample_in_val   : std_logic :=  '0';
signal      sample_out_val   : std_logic;

begin

BUTTERFLY_TOP1  :   BUTTERFLY_TOP
generic map(
    Sample_SZ    => Sample_SZ
)
port map(
    rstn    => rstn,
    clk     => clk,

    sample_in_val   => sample_in_val,
    sample_out_val  => sample_out_val,

    Are             => Are,
    Aim             => Aim,
    Bre             => Bre,
    Bim             => Bim,
    c_in            => c,
    cps_in          => cps,
    cms_in          => cms,

    butterfly_out   => Resultat,
    sel_out         => sel_out
);

clk     <=  not clk after 25 ns;

process
begin

if rstn = '0' then
    wait for 40 ns;
        rstn <= '1';
end if;

wait for 11 ns;
    Are <= std_logic_vector(TO_SIGNED(100   ,Sample_SZ));
    Aim <= std_logic_vector(TO_SIGNED(110   ,Sample_SZ));
    Bre <= std_logic_vector(TO_SIGNED(-40   ,Sample_SZ));
    Bim <= std_logic_vector(TO_SIGNED(10    ,Sample_SZ));
    c   <= std_logic_vector(TO_SIGNED(121   ,Sample_SZ));
    cps <= std_logic_vector(TO_SIGNED(160   ,Sample_SZ));
    cms <= std_logic_vector(TO_SIGNED(82    ,Sample_SZ));

wait for 50 ns;
    sample_in_val   <= '1';

end process;

end TestBench_BUTTERFLY_TOP;
